#include <iostream>
#include <fstream>
#include <algorithm>
#include <boost/random/uniform_01.hpp>
#include <boost/random/mersenne_twister.hpp>
#include "random_acyclic_graph.hpp"
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/graphviz.hpp>
#define HASH
#include "topological_sort.hpp"

const int N = 1000;
const double edge_probability = 0.01;
const unsigned int randseed = 29u;
const bool output_graph = false;

using namespace boost;

int main()
{
  typedef mt19937 RandGen;
  RandGen gen1;
  gen1.seed(31u);
  uniform_01<RandGen> rand_gen(gen1);

  typedef adjacency_list<listS, vecS, directedS> graph_type;
  graph_type g;

  std::cout << "Generating a random graph with " << N << " vertices"
	    << " and about " << static_cast<int>((N*(N-1)/2)*edge_probability) 
	    << " edges" << std::endl;
  generate_random_acyclic_graph(g, N, edge_probability, rand_gen);

#if 0
  graph_traits<graph_type>::vertex_iterator vi, vi_end;
  for (tie(vi, vi_end) = vertices(g); vi != vi_end; ++vi) {
    std::cout << "Vertex " << *vi << " has edges to " << std::endl;
    graph_traits<graph_type>::adjacency_iterator avi, avi_end;
    for (tie(avi, avi_end) = adjacent_vertices(*vi, g);
	 avi != avi_end; ++avi)
      std::cout << "   " << *avi << std::endl;
  }
#endif
  
  std::cout << "The random graph has " << num_vertices(g) << " vertices"
	    << " and " << num_edges(g) << " edges" << std::endl;

  if (output_graph) {
    std::ofstream dot_file("randgraph.dot");
    write_graphviz(dot_file, g);
  }

  typedef graph_traits<graph_type>::vertex_descriptor vertex_t;
  std::vector<vertex_t> order;
  
  std::cout << "Topological sorting ..." << std::endl;
  topological_sort(g, std::back_inserter(order));

  if (output_graph) {
    std::cout << "The ordering produced:\n";
    std::copy(order.begin(), order.end(), 
	      std::ostream_iterator<vertex_t>(std::cout, " "));
    std::cout << std::endl;
  }
  
  std::cout << "Checking the result ..." << std::endl;
  if (check_topological_sort(g, order.begin(), order.end()))
    std::cout << "Result passes the acceptance test" << std::endl;
  else
    std::cout << "Result doesn't pass the acceptance test!" << std::endl;

  return 0;
}
