#include <iostream>
#include <fstream>
#include <algorithm>
#include <boost/graph/adjacency_list.hpp>
#define HASH
#include "topological_sort.hpp"


using namespace boost;

typedef adjacency_list<setS, listS, directedS> file_dep_graph;

namespace std {
  template <typename T>
  std::istream& operator>>(std::istream& in, std::pair<T,T>& p)
  { 
    in >> p.first >> p.second;
    return in;
  }
}

int main()
{
  typedef graph_traits<file_dep_graph>::vertex_descriptor vertex_t;
  std::ifstream file_in("makefile-dependencies.dat");
  typedef graph_traits<file_dep_graph>::vertices_size_type 
    size_type;
  size_type n_vertices;
  file_in >> n_vertices;  // read in number of vertices
  
  std::cout << "Number of vertices: " << n_vertices << std::endl;

  typedef std::pair<std::size_t,std::size_t> Pair;
  std::istream_iterator<Pair> input_begin(file_in), input_end;
  
  file_dep_graph g(input_begin, input_end, n_vertices);

  std::map<vertex_t, std::string> name;
  std::ifstream name_in("makefile-target-names.dat");
  graph_traits<file_dep_graph>::vertex_iterator vi, vi_end;
  for (tie(vi, vi_end) = vertices(g); vi != vi_end; ++vi)
    name_in >> name[*vi];
  
  // Added by DM: output the graph
  for (tie(vi, vi_end) = vertices(g); vi != vi_end; ++vi) {
    std::cout << "File " << name[*vi] << " is used by: " << std::endl;
    graph_traits<file_dep_graph>::adjacency_iterator avi, avi_end;
    for (tie(avi, avi_end) = adjacent_vertices(*vi, g);
	 avi != avi_end; ++avi)
      std::cout << "   " << name[*avi] << std::endl;
  }

  std::vector<vertex_t> order;
  
  topological_sort(g, back_inserter(order));
  
  std::cout << "\nA topological ordering: " << std::endl;
  for (int i = 0; i < num_vertices(g); ++i)
    std::cout << name[order[i]] << std::endl;
  std::cout << std::endl;

  if (check_topological_sort(g, order.begin(), order.end()))
    std::cout << "Result passes the acceptance test" << std::endl;
  else
    std::cout << "Result doesn't pass the acceptance test!" << std::endl;
  return 0;
}
